/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.relevance.vehicular;

import de.tudarmstadt.maki.simonstrator.api.component.HostComponent;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.information.VehicularPointInformation;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.RoadNetworkEdge;

/**
 * This interface provide methods to calculate the relevance of an event for a
 * vehicle.
 * 
 * @author Tobias Meuser
 * @version 1.0.0
 *
 */
public interface VehicularRelevanceCalculationComponent extends HostComponent, RelevanceCalculationComponent {
	public enum PositionRepresentation {
		ROAD_BASED, LOCATION_BASED
	}

	PositionRepresentation getPositionRepresentation();

	/**
	 * This method calculates the relevance of an event for a vehicle, combining the
	 * temporal and the geographical relevance.
	 * 
	 * @param pVehiclePosition
	 *            The vehicle's position
	 * @param pInformation
	 *            The information for which the relevance is calculated.
	 * @return The relevance of the event.
	 */
	double calculateRelevance(RoadNetworkEdge pVehiclePosition, VehicularPointInformation pInformation);

	/**
	 * This method calculates the relevance of an event for a vehicle, combining the
	 * temporal and the geographical relevance.
	 * 
	 * @param pVehiclePosition
	 *            The vehicle's position
	 * @param pInformation
	 *            The information for which the relevance is calculated.
	 * @return The relevance of the event.
	 */
	double calculateRelevance(Location pLocation, VehicularPointInformation pInformation);

	/**
	 * This method calculates the relevance of an event for a vehicle, combining
	 * the temporal and the geographical relevance.
	 * 
	 * @param pVehiclePosition
	 *            The vehicle's position
	 * @param pInformation
	 *            The information for which the relevance is calculated.
	 * @return The relevance of the event.
	 */
	double calculateRelevanceWithoutAnalyzer(RoadNetworkEdge pVehiclePosition, VehicularPointInformation pInformation);
}
