/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


package de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.jam;

import java.util.HashMap;
import java.util.Map;

import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.RoadProperty;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.generator.lifetime.LifetimeDistribution;

/**
 * @author Tobias Meuser (tobias.meuser@kom.tu-darmstadt.de)
 * @version 1.0 at 24.04.2018
 *
 */
public class EventInformationContainer {
	private static Map<Class<? extends RoadProperty>, Long> durations = new HashMap<>();
	private static Map<Class<? extends RoadProperty>, LifetimeDistribution> lifetimeDistribution = new HashMap<>();
	private static Map<Class<? extends RoadProperty>, Double> disseminationRange = new HashMap<>();

	public static void registerEventDuration(Class<? extends RoadProperty> pClass, long pDuration) {
		if (!durations.containsKey(pClass)) {
			durations.put(pClass, pDuration);
		} else {
			throw new AssertionError(pClass + " has already been registered!");
		}
	}

	public static void registerEventRange(Class<? extends RoadProperty> pClass, double pRange) {
		if (!disseminationRange.containsKey(pClass)) {
			disseminationRange.put(pClass, pRange);
		} else {
			throw new AssertionError(pClass + " has already been registered!");
		}
	}

	public static void registerEventLifetimeDistribution(Class<? extends RoadProperty> pClass,
			LifetimeDistribution pLifetimeDistribution) {
		if (!lifetimeDistribution.containsKey(pClass)) {
			lifetimeDistribution.put(pClass, pLifetimeDistribution);
		} else {
			throw new AssertionError(pClass + " has already been registered!");
		}
	}

	public static long getEventDuration(Class<? extends RoadProperty> pClass) {
		if (durations.containsKey(pClass)) {
			return durations.get(pClass);
		}
		throw new AssertionError(pClass + " has not been registered!");
	}

	public static double getEventRange(Class<? extends RoadProperty> pClass) {
		if (disseminationRange.containsKey(pClass)) {
			return disseminationRange.get(pClass);
		}
		return Double.POSITIVE_INFINITY;
	}

	public static LifetimeDistribution getEventLifetimeDistribution(Class<? extends RoadProperty> pClass) {
		if (lifetimeDistribution.containsKey(pClass)) {
			return lifetimeDistribution.get(pClass);
		}
		return null;
	}
}
