package de.tudarmstadt.maki.simonstrator.api.component.vehicular.api;

import java.util.List;

import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.RoadNetwork;

/**
 * 
 * @author Tobias Meuser (tobias.meuser@kom.tu-darmstadt.de)
 * @version 1.0 at 06.11.2017
 *
 */
public interface SimulationSetupExtractor {

	/**
	 * This method is used to restrict the shown vehicles to the vehicles in a
	 * certain area.
	 * 
	 * @param pStartX
	 *            the x-start of the observed area
	 * @param pStartY
	 *            the y-start of the observed area
	 * @param pEndX
	 *            the x-end of the observed area
	 * @param pEndY
	 *            the x-end of the observed area
	 */
	void setObservedArea(double pStartX, double pStartY, double pEndX, double pEndY);

	/**
	 * Initializes the controller
	 */
	void init(long pTimeScale);

	/**
	 * get all intersections on the map
	 * 
	 * @param pCluster
	 *            true, if close intersections shall be clustered
	 * @return the list of all intersections
	 */
	List<Location> getAllIntersections(boolean pCluster);

	/**
	 * get the width of the scenario
	 * 
	 * @return the width of the scenario
	 */
	double getScenarioWidth();

	/**
	 * get the height of the scenario
	 * 
	 * @return the height of the scenario
	 */
	double getScenarioHeight();

	/**
	 * get the road network
	 * 
	 * @return the road network
	 */
	RoadNetwork getRoadNetwork();
}
