/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.vehicular.caching.replacement;

import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.aggregation.AggregationInformation;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.RoadProperty;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.information.AggregatedInformation;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.information.PointInformation;

public class SimilarInformationReplacementStrategy implements CacheReplacementStrategy {

	@Override
	public boolean replaceInformation(PointInformation<?> pCachedInformation, PointInformation<?> pSensedInformation) {
		if (pCachedInformation instanceof AggregatedInformation
				&& pSensedInformation instanceof AggregatedInformation) {
			AggregatedInformation cachedAggregate = ((AggregatedInformation) pCachedInformation);
			AggregatedInformation sensedAggregate = ((AggregatedInformation) pSensedInformation);

			boolean same = true;
			if (pCachedInformation.getValue() instanceof RoadProperty
					&& pSensedInformation.getValue() instanceof RoadProperty) {
				RoadProperty cachedValue = (RoadProperty) pCachedInformation.getValue();
				RoadProperty sensedValue = (RoadProperty) pSensedInformation.getValue();

				same = cachedValue.getEdge().equals(sensedValue.getEdge());
			}
			
			if (same && cachedAggregate.isAggregated() && sensedAggregate.isAggregated()) {
				AggregationInformation cachedMetaInfo = cachedAggregate.getAggregationInformation();
				AggregationInformation sensedMetaInfo = sensedAggregate.getAggregationInformation();

				return sensedMetaInfo.getMinTimestamp() <= pCachedInformation.getDetectionDate();
			}
		}
		return false;
	}

}
