/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


package de.tudarmstadt.maki.simonstrator.api.component.vehicular.information;

import java.util.List;

import de.tudarmstadt.maki.simonstrator.api.Time;
import de.tudarmstadt.maki.simonstrator.api.common.Transmitable;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.aggregation.AggregationInformation;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.AggregatedProperty;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.RoadProperty;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.RoadNetworkEdge;

/**
 * @author Tobias Meuser (tobias.meuser@kom.tu-darmstadt.de)
 * @version 1.0 at 05.03.2018
 *
 */
public class RoadInformation extends EnvironmentInformation<RoadProperty>
		implements VehicularPointInformation<RoadProperty>, AggregatedInformation, Cloneable, Transmitable {
	private boolean _local = false;

	public RoadInformation(RoadProperty pEnvironment) {
		super(pEnvironment);
	}

	public void setLocal() {
		_local = true;
	}

	public boolean isLocal() {
		return _local;
	}

	@Override
	public RoadNetworkEdge getEdge() {
		return getValue().getEdge();
	}

	@Override
	public <T> void setAttribute(AvailableInformationAttributes pKey, T pValue) {
		if (AvailableInformationAttributes.EDGE.equals(pKey)) {
			getValue().setEdge((RoadNetworkEdge) pValue);
		} else {
			super.setAttribute(pKey, pValue);
		}
	}

	@Override
	public List<AvailableInformationAttributes> getAvailableAttributes() {
		List<AvailableInformationAttributes> result = super.getAvailableAttributes();

		result.add(AvailableInformationAttributes.EDGE);

		return result;
	}

	@Override
	public <T> T getAttribute(AvailableInformationAttributes pKey) {
		if (AvailableInformationAttributes.EDGE.equals(pKey)) {
			return (T) getEdge();
		}
		return super.getAttribute(pKey);
	}

	public void copyAttributes(RoadInformation pSrc) {
		for (AvailableInformationAttributes attribute : pSrc.getAvailableAttributes()) {
			setAttribute(attribute, pSrc.getAttribute(attribute));
		}
	}

	@Override
	public <T> boolean hasAttribute(AvailableInformationAttributes pKey) {
		return super.hasAttribute(pKey);
	}

	@Override
	public AggregationInformation getAggregationInformation() {
		if (getValue() instanceof AggregatedProperty) {
			return ((AggregatedProperty) getValue()).getAggregationInformation();
		}
		return null;
	}

	@Override
	public boolean isAggregated() {
		if (getValue() instanceof AggregatedProperty) {
			return ((AggregatedProperty) getValue()).isAggregated();
		}
		return false;
	}

	@Override
	public boolean isValid() {
		return getDetectionDate() + (long) getAttribute(AvailableInformationAttributes.TTL) > Time.getCurrentTime();
	}

	@Override
	public int hashCode() {
		return getEdge().hashCode() << 16 + Long.hashCode(getDetectionDate());
	}

	@Override
	public boolean equals(Object pObj) {
		if (pObj instanceof RoadInformation) {
			RoadInformation roadInfo = (RoadInformation) pObj;
			return getValue().equals(getValue()) && getDetectionDate() == roadInfo.getDetectionDate()
					&& (getLocation() == roadInfo.getLocation()
							|| (getLocation() != null && getLocation().equals(roadInfo.getLocation())))
					&& getEdge().equals(((RoadInformation) pObj).getEdge());
		}
		return false;
	}

	@Override
	public RoadInformation clone() {
		RoadInformation roadInformation = new RoadInformation(getValue().clone());
		for (AvailableInformationAttributes attribute : AvailableInformationAttributes.values()) {
			if (hasAttribute(attribute)) {
				roadInformation.setAttribute(attribute, getAttribute(attribute));
			}
		}
		roadInformation._local = _local;
		return roadInformation;
	}

	@Override
	public int getTransmissionSize() {
		return 250;
	}
}
