/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 *
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.paths.VehiclePathTracker;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.paths.VehiclePathTrackerFactory;

public class RoadNetworkRoute implements Iterable<RoadNetworkEdge> {
	private List<RoadNetworkEdge> _route;

	public RoadNetworkRoute(List<RoadNetworkEdge> pRoute) {
		_route = pRoute;
	}

	public RoadNetworkRoute(List<String> pRoute, RoadNetwork pNetwork) {
		_route = new ArrayList<>();

		for (String pRouteID : pRoute) {
			RoadNetworkEdge edge = pNetwork.getEdge(pRouteID);
			_route.add(edge);
		}
	}

	public List<RoadNetworkEdge> getRoute() {
		return _route;
	}

	@Override
	public boolean equals(Object pObj) {
		if (pObj instanceof RoadNetworkRoute) {
			List<RoadNetworkEdge> route = ((RoadNetworkRoute) pObj).getRoute();

			if (route.size() == _route.size()) {
				for (int i = 0; i < route.size(); i++) {
					if (!route.get(i).equals(_route.get(i))) {
						return false;
					}
				}
				return true;
			}
		}
		return false;
	}

	public boolean containsEdge(String pEdgeID) {
		for (RoadNetworkEdge roadNetworkEdge : _route) {
			if (roadNetworkEdge.getEdgeID().equals(pEdgeID)) {
				return true;
			}
		}
		return false;
	}

	public boolean containsEdge(RoadNetworkEdge pEdge) {
		return _route.contains(pEdge);
	}

	public RoadNetworkEdge getStart() {
		return _route.get(0);
	}

	public RoadNetworkEdge getDestination() {
		return _route.get(_route.size() - 1);
	}

	@Override
	public String toString() {
		StringBuffer buffer = new StringBuffer();
		for (int i = 0; i < _route.size(); i++) {
			if (i != 0) {
				buffer.append(" --> ");
			}
			buffer.append(_route.get(i));
		}
		return buffer.toString();
	}

	public boolean containsEdges(List<RoadNetworkEdge> pBlocked) {
		for (RoadNetworkEdge roadNetworkEdge : pBlocked) {
			if (containsEdge(roadNetworkEdge)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * @return
	 */
	public double getTravelTime() {
		VehiclePathTracker tracker = VehiclePathTrackerFactory.getVehiclePathTracker();
		double time = 0;
		for (int i = 0; i < _route.size() - 1; i++) {
			time += tracker.getTravelTime(_route.get(i), _route.get(i + 1));
		}
		time += _route.get(_route.size() - 1).getLength() / _route.get(_route.size() - 1).getMaxSpeed();
		return time;
	}

	/**
	 * @param pEdge
	 * @param pEdge2
	 * @return
	 */
	public RoadNetworkRoute subRoute(RoadNetworkEdge pStart, RoadNetworkEdge pEnd) {
		List<RoadNetworkEdge> edges = new ArrayList<>();

		boolean startFound = false;
		boolean endFound = false;
		for (RoadNetworkEdge roadNetworkEdge : _route) {
			if (roadNetworkEdge.equals(pStart)) {
				startFound = true;
			}
			if (startFound ^ endFound) {
				edges.add(roadNetworkEdge);
			}
			if (roadNetworkEdge.equals(pEnd)) {
				endFound = true;
			}
		}
		return new RoadNetworkRoute(edges);
	}

	@Override
	public Iterator<RoadNetworkEdge> iterator() {
		return _route.iterator();
	}

	public double calculateStandardRouteCosts() {
		double sum = 0;
		for (RoadNetworkEdge roadNetworkEdge : _route) {
			sum += roadNetworkEdge.calculateStandardEdgeCosts();
		}
		return sum;
	}

	public double calculateDrivingDurationTo(RoadNetworkEdge pDestination) {
		double sum = 0;
		for (int i = 1; i < _route.size(); i++) {
			if (!_route.get(i).equals(pDestination)) {
				sum += _route.get(i).calculateStandardEdgeCosts();
			} else {
				break;
			}
		}
		return sum;
	}
}
