/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 *
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.xml.bind.annotation.XmlElement;

public class SerializableRoadNetworkEdge implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = -311466941428215723L;

	@XmlElement(name = "edgeID")
	private String _edgeID;
	@XmlElement(name = "angle")
	private double _angle;

	@XmlElement(name = "lanes")
	private List<SerializableRoadNetworkLane> _lanes = new ArrayList<>();

	@XmlElement(name = "maxSpeed")
	private double _maxSpeed = 0;

	private List<String> _accessibleEdgeIDs = new ArrayList<>();

	private SerializableRoadNetworkEdge() {
	}

	public SerializableRoadNetworkEdge(RoadNetworkEdge pEdge) {
		_edgeID = pEdge.getEdgeID();
		_angle = pEdge.getAngle();

		for (RoadNetworkLane lane : pEdge.getLanes()) {
			_lanes.add(new SerializableRoadNetworkLane(lane));
		}

		_maxSpeed = pEdge.getOriginalMaxSpeed();

		for (RoadNetworkEdge edge : pEdge.getAccessibleEdges()) {
			_accessibleEdgeIDs.add(edge.getEdgeID());
		}
	}

	public String getEdgeID() {
		return _edgeID;
	}

	public double getAngle() {
		return _angle;
	}

	public double getMaxSpeed() {
		return _maxSpeed;
	}

	public int getLaneAmount() {
		return _lanes.size();
	}

	public void setMaxSpeed(double pMaxSpeed) {
		if (pMaxSpeed > _maxSpeed) {
			_maxSpeed = pMaxSpeed;
		}
	}

	public List<String> getAccessibleEdges() {
		return Collections.unmodifiableList(_accessibleEdgeIDs);
	}

	public String getDescription() {
		return "id: " + _edgeID + ", lanes: " + _lanes.size() + ", speed limit: " + _maxSpeed;
	}

	@Override
	public String toString() {
		return _edgeID;
	}

	public boolean isInternal() {
		return _edgeID.startsWith(":");
	}

	public List<SerializableRoadNetworkLane> getLanes() {
		return _lanes;
	}
}
