package de.tudarmstadt.maki.simonstrator.api.util;
/*
 * Copyright (c) 2005-2010 KOM � Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */




import java.util.ArrayList;
import java.util.List;

import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;

/**
 * @author Tobias Meuser (tobias.meuser@kom.tu-darmstadt.de)
 * @version 1.0 at 02.03.2018
 *
 */
public class Geohash {
	public static final int MAX_PRECISION = 12;

	public static String encode(Location pLocation, int pPrecision) {
		byte[] bytes = new byte[pPrecision];

		double offsetX = 0;
		double offsetY = 0;
		for (int i = 0; i < pPrecision; i++) {
		    for (int j = 0; j < 5; j++) {
		        Area cellSize = getCellSize(i + 1, j);

		        double currentCellX = pLocation.getLongitudeOrX() - offsetX;
		        double currentCellY = pLocation.getLatitudeOrY() - offsetY;

		        boolean nextBit;
				if ((i + j) % 2 == 0) {
		            nextBit = getNextBit(currentCellX, cellSize.getSizeX());
		        } else {
		            nextBit = getNextBit(currentCellY, cellSize.getSizeY());
		        }

		        if (nextBit) {
					if ((i + j) % 2 == 0) {
		                offsetX += cellSize.getSizeX() / 2.0;
		            } else {
		                offsetY += cellSize.getSizeY() / 2.0;
		            }
		        }

		        int value;
		        if (nextBit) {
		            value = 1;
		        } else {
		            value = 0;
		        }
		        
		        bytes[i] += value * Math.pow(2, 4 - j);
            }
		    
        }

		return encode(bytes);
	}
	
	public static String[] getAdjacentGeohashes(String pGeohash) {
		String[] geohashes = new String[] { getAdjacentGeohash(pGeohash, 1, 0), getAdjacentGeohash(pGeohash, -1, 0),
				getAdjacentGeohash(pGeohash, 0, 1), getAdjacentGeohash(pGeohash, 0, -1) };

		return geohashes;
	}

	/**
	 * @param pGeohash
	 * @param pX
	 * @param pY
	 * @return
	 */
	public static String getAdjacentGeohash(String pGeohash, int pX, int pY) {
		/*
		 * (1,0) right/eastern GeoHash (-1,0) left/western GeoHash (0,1)
		 * lower/southernGeoHash (0,-1) upper/northern GeoHash
		 */
		byte[] decode = decode(pGeohash);

		if (pX != 0) {
			boolean reduce = pX < 0;
			outer: for (int i = decode.length - 1; i >= 0; i--) {
				for (int j = 0; j < 5; j++) {
					if ((i + j) % 2 == 0) {
						boolean bitSet = isBitSet(decode[i], j);
						if (reduce) {
							decode[i] = toogleBit(decode[i], j);
							if (bitSet) {
								break outer;
							}
						} else {
							decode[i] = toogleBit(decode[i], j);
							if (!bitSet) {
								break outer;
							}
						}
					}
				}
			}
		}

		if (pY != 0) {
			boolean reduce = pY < 0;
			outer: for (int i = decode.length - 1; i >= 0; i--) {
				for (int j = 0; j < 5; j++) {
					if ((i + j) % 2 == 1) {
						boolean bitSet = isBitSet(decode[i], j);
						if (reduce) {
							decode[i] = toogleBit(decode[i], j);
							if (bitSet) {
								break outer;
							}
						} else {
							decode[i] = toogleBit(decode[i], j);
							if (!bitSet) {
								break outer;
							}
						}
					}
				}
			}
		}

		return encode(decode);
	}

	/**
	 * Get the adjacent regions depending if vehicle drives in N, E, S, W direction
	 * 
	 * @param geohash
	 *            center geohash
	 * @param heading
	 *            angle of driving direction
	 * @return adjacent geohash regions depending on driving direction
	 */
	public static List<String> getSubscriptionHorizon(String geohash, double heading) {
		List<String> adjacentAreas = new ArrayList<>();
		int direction = (int) heading;

		if (0 <= direction && direction < 45 || 315 <= direction && direction <= 360) {// N
			adjacentAreas.add(getAdjacentGeohash(geohash, 0, -1));// top
			adjacentAreas.add(getAdjacentGeohash(geohash, 1, 0));// right
			adjacentAreas.add(getAdjacentGeohash(geohash, -1, 0));// left
			adjacentAreas.add(getAdjacentGeohash(getAdjacentGeohash(geohash, 0, -1), 1, 0));// top,right
			adjacentAreas.add(getAdjacentGeohash(getAdjacentGeohash(geohash, 0, -1), -1, 0));// top,left
		}

		if (45 <= direction && direction < 135) {// E
			adjacentAreas.add(getAdjacentGeohash(geohash, 1, 0));// right
			adjacentAreas.add(getAdjacentGeohash(geohash, 0, -1));// top
			adjacentAreas.add(getAdjacentGeohash(geohash, 0, 1));// bottom
			adjacentAreas.add(getAdjacentGeohash(getAdjacentGeohash(geohash, 1, 0), 0, -1));// right, top
			adjacentAreas.add(getAdjacentGeohash(getAdjacentGeohash(geohash, 1, 0), 0, 1));// right, bottom
		}

		if (135 <= direction && direction < 225) {// S
			adjacentAreas.add(getAdjacentGeohash(geohash, 0, 1));// bottom
			adjacentAreas.add(getAdjacentGeohash(geohash, 1, 0));// right
			adjacentAreas.add(getAdjacentGeohash(geohash, -1, 0));// left
			adjacentAreas.add(getAdjacentGeohash(getAdjacentGeohash(geohash, 0, 1), 1, 0));// bottom, right
			adjacentAreas.add(getAdjacentGeohash(getAdjacentGeohash(geohash, 0, 1), -1, 0));// bottom, left

		}
		if (225 <= direction && direction < 315) {// W
			adjacentAreas.add(getAdjacentGeohash(geohash, -1, 0));// left
			adjacentAreas.add(getAdjacentGeohash(geohash, 0, 1));// bottom
			adjacentAreas.add(getAdjacentGeohash(geohash, 0, -1));// top
			adjacentAreas.add(getAdjacentGeohash(getAdjacentGeohash(geohash, -1, 0), 0, -1));// left, top
			adjacentAreas.add(getAdjacentGeohash(getAdjacentGeohash(geohash, -1, 0), 0, 1));// left, bottom
		}
		return adjacentAreas;
	}

	private static boolean isBitSet(byte b, int offset) {
		return getBit(b, offset) == 1;
	}

	private static byte getBit(byte b, int offset) {
		return (byte) ((b % Math.pow(2, offset + 1)) / Math.pow(2, offset));
	}

	private static byte setBit(byte b, int offset, int value) {
		return (byte) (b + Math.pow(2, offset) * (value - getBit(b, offset)));
	}

	private static byte toogleBit(byte b, int offset) {
		if (isBitSet(b, offset)) {
			return setBit(b, offset, 0);
		} else {
			return setBit(b, offset, 1);
		}
	}

	private static String encode(byte[] pBytes) {
		StringBuffer buffer = new StringBuffer();
		for (int i = 0; i < pBytes.length; i++) {
			byte b = pBytes[i];
			buffer.append(encodeByte(b));
		}
		return buffer.toString();
	}

	private static char encodeByte(byte b) {
		if (b <= 9) {
			return (char) (b + '0');
		} else if (b <= 16) {
			return (char) (b + 'b' - 10);
		} else if (b <= 18) {
			return (char) (b + 'j' - 17);
		} else if (b <= 20) {
			return (char) (b + 'm' - 19);
		} else if (b <= 32) {
			return (char) (b + 'p' - 21);
		} else {
			throw new IllegalArgumentException("Must not have a value greater than 32 in Base32 encoding!");
		}
	}

	private static byte decodeByte(char c) {
		if (c >= 'p') {
			return (byte) (c - 'p' + 21);
		} else if (c >= 'm') {
			return (byte) (c - 'm' + 19);
		} else if (c >= 'j') {
			return (byte) (c - 'j' + 17);
		} else if (c >= 'b') {
			return (byte) (c - 'b' + 10);
		} else if (c >= '0') {
			return (byte) (c - '0' + 0);
		}
		return -1;
	}

	public static byte[] decode(String pGeohash) {
		byte[] bytes = new byte[pGeohash.length()];

		for (int i = 0; i < pGeohash.length(); i++) {
			bytes[i] = decodeByte(pGeohash.charAt(i));
		}

		return bytes;
	}

	private static boolean getNextBit(double currentCellPosition, double currentCellSize) {
	    double border = currentCellSize / 2.0;
        
        if (currentCellPosition > border) {
            return true;
        } else {
            return false;
        }
	}
    
    public static double getAreaWidth(int pPrecision) {
        return getCellSize(pPrecision).getSizeX();
    }
    
    public static double getAreaHeight(int pPrecision) {
        return getCellSize(pPrecision).getSizeY();
    }

    private static Area getCellSize(int pPrecision) {
        return getCellSize(pPrecision, 4);
    }

    private static Area getCellSize(int pPrecision, int pStep) {
        double sizeX = 20000000;
        double sizeY = 20000000;
        
        for (int i = 1; i <= (pPrecision - 1) * 5 + pStep; i++) {
            if (i % 2 == 0) {
                sizeX *= 0.5;
            } else {
                sizeY *= 0.5;
            }
        }
        
        return new Area(sizeX, sizeY);
    }
	
	private static class Area {
	    private double sizeX;
	    private double sizeY;
	    
        public Area(double pSizeX, double pSizeY) {
            sizeX = pSizeX;
            sizeY = pSizeY;
        }

        public double getSizeX() {
            return sizeX;
        }

        public double getSizeY() {
            return sizeY;
        }
        
        @Override
        public String toString() {
            return "(" + sizeX + "|" + sizeY + ")";
        }
	}

}
