package de.tudarmstadt.maki.simonstrator.api.web;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlID;
import javax.xml.bind.annotation.XmlTransient;

import de.tudarmstadt.maki.simonstrator.api.web.approach.Approach;
import de.tudarmstadt.maki.simonstrator.api.web.parameter.Parameter;
import de.tudarmstadt.maki.simonstrator.api.web.plotting.BoxPlot;
import de.tudarmstadt.maki.simonstrator.api.web.plotting.Plot;
import de.tudarmstadt.maki.simonstrator.api.web.plotting.TimePlot;

@XmlAccessorType(XmlAccessType.FIELD)
public class Scenario {
	@XmlID
    @XmlElement(name = "name")
    private String _name;

    @XmlElement(name = "config")
    private String _config;

    @XmlElementWrapper(name = "approaches")
    @XmlElement(name = "approach")
    List<Approach> _approaches = new ArrayList<>();

    @XmlElementWrapper(name = "parameters")
    @XmlElement(name = "parameter")
    List<Parameter> _parameters = new ArrayList<>();

	@XmlElementWrapper(name = "plots")
	@XmlElement(name = "plot")
	List<Plot> _plots = new ArrayList<>();

	@XmlTransient
	private int _currentID = -1;

    public Scenario() {
    }

    public Scenario(String pName, String pConfig) {
        _name = pName;
        _config = pConfig;
    }

    public void setName(String pName) {
        _name = pName;
    }

    public void setConfig(String pConfig) {
        _config = pConfig;
    }

    public String getName() {
        return _name;
    }

    public String getConfig() {
        return _config;
    }

    public void addApproach(Approach pApproach) {
        _approaches.add(pApproach);
    }

    public List<Approach> getApproaches() {
        return _approaches;
    }

    public void addParameter(Parameter pParameter) {
        _parameters.add(pParameter);
    }

	public void addPlot(Plot pPlot) {
		_plots.add(pPlot);
	}

	public Plot createBoxPlot() {
		if (_currentID == -1) {
			for (Plot plot : _plots) {
				if (plot.getPlotID() > _currentID) {
					_currentID = plot.getPlotID();
				}
			}
		}
		return new BoxPlot(++_currentID);
	}

	public Plot createTimePlot() {
		if (_currentID == -1) {
			for (Plot plot : _plots) {
				if (plot.getPlotID() > _currentID) {
					_currentID = plot.getPlotID();
				}
			}
		}
		return new TimePlot(++_currentID);
	}

	public Plot createPlot(Class<? extends Plot> pClass) {
		if (_currentID == -1) {
			for (Plot plot : _plots) {
				if (plot.getPlotID() > _currentID) {
					_currentID = plot.getPlotID();
				}
			}
		}

		try {
			Constructor<? extends Plot> constructor = pClass.getConstructor(Integer.TYPE);
			return constructor.newInstance(++_currentID);
		} catch (NoSuchMethodException | SecurityException | InstantiationException | IllegalAccessException
				| IllegalArgumentException | InvocationTargetException e) {
			e.printStackTrace();
		}
		return null;
	}

    public List<Parameter> getParameters() {
        return _parameters;
    }

    public Approach getApproach(String pName) {
        for (Approach approach : _approaches) {
            if (approach.getName().equals(pName)) {
                return approach;
            }
        }
        return null;
    }

    public Parameter getParameter(String pName) {
        for (Parameter parameter : _parameters) {
            if (parameter.getName().equals(pName)) {
                return parameter;
            }
        }
        return null;
    }

	public Plot getPlot(int pPlotID) {
		for (Plot plot : _plots) {
			if (plot.getPlotID() == pPlotID) {
				return plot;
			}
		}
		return null;
	}

	public List<Plot> getPlots() {
		return _plots;
	}

	public void removePlot(int pPlotID) {
		for (Plot plot : _plots) {
			if (plot.getPlotID() == pPlotID) {
				_plots.remove(plot);
				return;
			}
		}
	}
}